@@ -3,6 +3,7 @@ import base64
 import json
 from urllib.parse import urlencode
 from unittest.mock import Mock
+from urllib.parse import urlparse
 
 from stellar_sdk.keypair import Keypair
 from stellar_sdk.transaction_envelope import TransactionEnvelope
@@ -42,8 +43,8 @@ def test_auth_get_account(client):
 
     manage_data_op = transaction_object.operations[0]
     assert manage_data_op.type_code() == Xdr.const.MANAGE_DATA
-    assert manage_data_op.data_name == "SEP 24 Reference auth"
-    assert len(manage_data_op.data_value) == 64
+    assert manage_data_op.data_name == f"{urlparse(settings.HOST_URL).hostname} auth"
+    assert len(manage_data_op.data_value) <= 64
     assert len(base64.b64decode(manage_data_op.data_value)) == 48
 
     signatures = envelope_object.signatures
